@@ -29,7 +29,7 @@ use crate::{
             TraitMethodDetails,
         },
         api::UnsafetyNeeded,
-        codegen_rs::lifetime::add_lifetime_to_all_params,
+        codegen_rs::lifetime::add_lifetime_to_all_reference_params,
     },
     types::{Namespace, QualifiedName},
 };
@@ -97,7 +97,7 @@ pub(super) fn gen_function(
     let params = analysis.params;
     let vis = analysis.vis;
     let kind = analysis.kind;
-    let doc_attr = fun.doc_attr;
+    let doc_attrs = fun.doc_attrs;
 
     let mut cpp_name_attr = Vec::new();
     let mut impl_entry = None;
@@ -113,7 +113,7 @@ pub(super) fn gen_function(
         rust_name,
         unsafety: &analysis.requires_unsafe,
         always_unsafe_due_to_trait_definition,
-        doc_attr: &doc_attr,
+        doc_attrs: &doc_attrs,
         non_pod_types,
     };
     // In rare occasions, we might need to give an explicit lifetime.
@@ -202,7 +202,7 @@ pub(super) fn gen_function(
     let extern_c_mod_item = ForeignItem::Fn(parse_quote!(
         #(#namespace_attr)*
         #(#cpp_name_attr)*
-        #doc_attr
+        #(#doc_attrs)*
         #vis #bridge_unsafety fn #cxxbridge_name #lifetime_tokens ( #params ) #ret_type;
     ));
     RsCodegenResult {
@@ -223,7 +223,7 @@ struct FnGenerator<'a> {
     rust_name: &'a str,
     unsafety: &'a UnsafetyNeeded,
     always_unsafe_due_to_trait_definition: bool,
-    doc_attr: &'a Option<Attribute>,
+    doc_attrs: &'a Vec<Attribute>,
     non_pod_types: &'a HashSet<QualifiedName>,
 }
 
@@ -231,7 +231,7 @@ impl<'a> FnGenerator<'a> {
     fn generate_arg_lists(
         &self,
         avoid_self: bool,
-    ) -> (Punctuated<FnArg, Comma>, Vec<TokenStream>, Vec<TokenStream>) {
+    ) -> (Punctuated<FnArg, Comma>, TokenStream, Vec<TokenStream>) {
         let mut wrapper_params: Punctuated<FnArg, Comma> = Punctuated::new();
         let mut local_variables = Vec::new();
         let mut arg_list = Vec::new();
@@ -253,6 +253,7 @@ impl<'a> FnGenerator<'a> {
             arg_list.push(actual_arg);
             local_variables.extend(local_variable.into_iter());
         }
+        let local_variables = quote! { #(#local_variables);* };
         (wrapper_params, local_variables, arg_list)
     }
 
@@ -272,16 +273,16 @@ impl<'a> FnGenerator<'a> {
         );
         let rust_name = make_ident(self.rust_name);
         let unsafety = self.unsafety.wrapper_token();
-        let doc_attr = self.doc_attr;
+        let doc_attrs = self.doc_attrs;
         let cxxbridge_name = self.cxxbridge_name;
         let call_body = self.wrap_call_with_unsafe(quote! {
             cxxbridge::#cxxbridge_name ( #(#arg_list),* )
         });
         Box::new(ImplBlockDetails {
             item: ImplItem::Method(parse_quote! {
-                #doc_attr
+                #(#doc_attrs)*
                 pub #unsafety fn #rust_name #lifetime_tokens ( #wrapper_params ) #ret_type {
-                    #(#local_variables),*
+                    #local_variables
                     #call_body
                 }
             }),
@@ -306,7 +307,7 @@ impl<'a> FnGenerator<'a> {
             ret_type,
             self.non_pod_types,
         );
-        let doc_attr = self.doc_attr;
+        let doc_attrs = self.doc_attrs;
         let unsafety = self.unsafety.wrapper_token();
         let cxxbridge_name = self.cxxbridge_name;
         let key = details.trt.clone();
@@ -315,9 +316,9 @@ impl<'a> FnGenerator<'a> {
             cxxbridge::#cxxbridge_name ( #(#arg_list),* )
         });
         let item = parse_quote! {
-            #doc_attr
+            #(#doc_attrs)*
             #unsafety fn #method_name #lifetime_tokens ( #wrapper_params ) #ret_type {
-                #(#local_variables),*
+                #local_variables
                 #call_body
             }
         };
@@ -354,25 +355,25 @@ impl<'a> FnGenerator<'a> {
         let rust_name = make_ident(&self.rust_name);
         let any_references = self.param_details.iter().any(|pd| pd.was_reference);
         let (lifetime_param, lifetime_addition) = if any_references {
-            add_lifetime_to_all_params(&mut wrapper_params);
+            add_lifetime_to_all_reference_params(&mut wrapper_params);
             (quote! { <'a> }, quote! { + 'a })
         } else {
             (quote! {}, quote! {})
         };
         let cxxbridge_name = self.cxxbridge_name;
         let body = quote! {
-            #(#local_variables),*
+            #local_variables
             autocxx::moveit::new::by_raw(move |#ptr_arg_name| {
                 let #ptr_arg_name = #ptr_arg_name.get_unchecked_mut().as_mut_ptr();
                 cxxbridge::#cxxbridge_name(#(#arg_list),* )
             })
         };
         let body = self.wrap_call_with_unsafe(body);
-        let doc_attr = self.doc_attr;
+        let doc_attrs = self.doc_attrs;
         let unsafety = self.unsafety.wrapper_token();
         Box::new(ImplBlockDetails {
             item: ImplItem::Method(parse_quote! {
-                #doc_attr
+                #(#doc_attrs)*
                 pub #unsafety fn #rust_name #lifetime_param ( #wrapper_params ) -> impl autocxx::moveit::new::New<Output=Self> #lifetime_addition {
                     #body
                 }
@@ -385,16 +386,16 @@ impl<'a> FnGenerator<'a> {
     fn generate_function_impl(&self, ret_type: &ReturnType) -> Item {
         let (wrapper_params, local_variables, arg_list) = self.generate_arg_lists(false);
         let rust_name = make_ident(self.rust_name);
-        let doc_attr = self.doc_attr;
+        let doc_attrs = self.doc_attrs;
         let unsafety = self.unsafety.wrapper_token();
         let cxxbridge_name = self.cxxbridge_name;
         let body = self.wrap_call_with_unsafe(quote! {
             cxxbridge::#cxxbridge_name ( #(#arg_list),* )
         });
         Item::Fn(parse_quote! {
-            #doc_attr
+            #(#doc_attrs)*
             pub #unsafety fn #rust_name ( #wrapper_params ) #ret_type {
-                #(#local_variables),*
+                #local_variables
                 #body
             }
         })
