@@ -45,7 +45,7 @@ pub enum ConvertError {
     UnknownDependentType(QualifiedName),
     IgnoredDependent(HashSet<QualifiedName>),
     ReservedName(String),
-    DuplicateCxxBridgeName,
+    DuplicateCxxBridgeName(Vec<String>),
     UnsupportedReceiver,
     BoxContainingNonRustType(QualifiedName),
     RustTypeWithAPath(QualifiedName),
@@ -99,7 +99,7 @@ impl Display for ConvertError {
             ConvertError::UnknownDependentType(qn) => write!(f, "This item relies on a type not known to autocxx ({})", qn.to_cpp_name())?,
             ConvertError::IgnoredDependent(qns) => write!(f, "This item depends on some other type(s) which autocxx could not generate, some of them are: {}", qns.iter().join(", "))?,
             ConvertError::ReservedName(id) => write!(f, "The item name '{}' is a reserved word in Rust.", id)?,
-            ConvertError::DuplicateCxxBridgeName => write!(f, "This item name is used in multiple namespaces. At present, autocxx and cxx allow only one type of a given name. This limitation will be fixed in future.")?,
+            ConvertError::DuplicateCxxBridgeName(items) => write!(f, "This item name is used in multiple namespaces. At present, autocxx and cxx allow only one type of a given name. This limitation will be fixed in future. (Items found with this name: {})", items.iter().join(", "))?,
             ConvertError::UnsupportedReceiver => write!(f, "This is a method on a type which can't be used as the receiver in Rust (i.e. self/this). This is probably because some type involves template specialization.")?,
             ConvertError::BoxContainingNonRustType(ty) => write!(f, "A rust::Box<T> was encountered where T was not known to be a Rust type. Use rust_type!(T): {}", ty.to_cpp_name())?,
             ConvertError::RustTypeWithAPath(ty) => write!(f, "A qualified Rust type was found (i.e. one containing ::): {}. Rust types must always be a simple identifier.", ty.to_cpp_name())?,
